{% ckan_extends %}

{% block resource_additional_information %}
    {% if pkg.doi %}
        {% snippet "doi/snippets/resource_citation.html", pkg_dict=pkg, res=res %}
    {% endif %}
    {{ super() }}
{% endblock %}
