{% ckan_extends %}

{# Author field is required for DOIs #}

{% block package_metadata_author %}

    {% block package_metadata_doi_author %}

        {{ form.input('author', label=_('Author(s)'), id='field-author', placeholder=_('Joe Bloggs'), value=data.author, error=errors.author, classes=['control-medium'], is_required=true) }}

        {% block package_metadata_author_email %}
            {{ form.input('author_email', label=_('Author Email'), id='field-author-email', placeholder=_('joe@example.com'), value=data.author_email, error=errors.author_email, classes=['control-medium']) }}
        {% endblock %}

    {% endblock %}

{% endblock %}
